require(vcfR)
#require(dummies)

## Arguments
## 1) input variant call file (vcf)
## 2) ensemblg gene file (gtf)
## 3) output file prefix
args <- commandArgs(trailingOnly = T)
# args <- c("ALL.chr1.phase3_shapeit2_mvncall_integrated_v5a.20130502.genotypes.vcf.gz",
    # "Homo_sapiens.GRCh38.102.gtf",
    # "test")

vcfFile <- args[1]

ensFile <- args[2]

vcfSel <- read.vcfR(vcfFile)

ens <- read.table(ensFile,
				  sep='\t',
				  colClasses=c("character", "character", "character", "integer", "integer", "character", "character", "character", "character"),
				  col.names=c("Chrom", "Source", "Feature", "Start", "Stop", "Score", "Strand", "Frame", "Info")
				  )
ens <- ens[ens$Chrom %in% as.character(1:22),]
ens$Chrom <- as.integer(ens$Chrom)
# filter genes
ens <- ens[ens$Feature == "gene",]
# filter protein coding
isProt <- sapply(1:nrow(ens), function(idx)
{
	s <- strsplit(ens[idx,]$Info, ';', fixed = T)[[1]]
	if (strsplit(s[5], " ", fixed = T)[[1]][3] == "protein_coding")
	{
		return(T)
	} else {
		return(F)
	}
})
ens <- ens[isProt,]
# Get gene names and ids
gene_info <- lapply(1:nrow(ens), function(idx)
{
	s <- strsplit(ens[idx,]$Info, ';', fixed = T)[[1]]
	s_id <- strsplit(s[1], " ", fixed = T)[[1]][2]
	s_name <- strsplit(s[3], " ", fixed = T)[[1]][3]
	return(list(id = s_id, name = s_name))
})
ens$id <- sapply(gene_info, function(x){x$id})
ens$name <- sapply(gene_info, function(x){x$name})


numSamples <- ncol(vcfSel@gt) - 1

# select rows which describe SNPs and are present in all samples and are do have frequency counts
rowSel <- (as.numeric(extract.info(vcfSel, element="NS")) == numSamples) &
	(extract.info(vcfSel, element="VT") == "SNP") &
	(!is.na(as.numeric(extract.info(vcfSel, element="AF"))))

vcfSel <- vcfSel[rowSel,]

rowSel2 <-(as.numeric(extract.info(vcfSel, element="AF")) > 0.01)

vcfSel <- vcfSel[rowSel2,]

rm(rowSel)
rm(rowSel2)


# filter variants in 1kb promoter region
promLen <- 1000
# Get only genes of this chromosome
ens <- ens[ens$Chrom == as.integer(getCHROM(vcfSel)[1]),]
ens$winStart <- sapply(1:nrow(ens), function(eIdx)
	{
		if (ens[eIdx,]$Strand == "+")
		{
			return(ens[eIdx,]$Start)
		} else {
			return(ens[eIdx,]$Stop - promLen)
		}
	})
ens$winEnd <- sapply(1:nrow(ens), function(eIdx)
	{
		if (ens[eIdx,]$Strand == "+")
		{
			return(ens[eIdx,]$Start + promLen)
		} else {
			return(ens[eIdx,]$Stop)
		}
	})
ens <- ens[order(ens$winStart),]
reordVcf <- order(getPOS(vcfSel))
vcfSel <- vcfSel[reordVcf,]
eIdx <- 1
promFilter <- rep(F, nrow(vcfSel))
geneIds <- rep("", nrow(vcfSel))
vIdx <- 1
while (vIdx  <= nrow(vcfSel) & eIdx <= nrow(ens))
{
	p <- getPOS(vcfSel[vIdx])
	if (p < ens[eIdx,]$winStart)
	{
		vIdx <- vIdx + 1
	} else {
		if (p >= ens[eIdx,]$winEnd)
		{
			eIdx <- eIdx + 1
		} else {
			promFilter[vIdx] <- T
			# test also following genes
			e2Idx <- eIdx
			gIdx <- vector()
			while (e2Idx <= nrow(ens) & p >= ens[e2Idx,]$winStart & p < ens[e2Idx,]$winEnd)
			{
				gIdx <- c(gIdx, e2Idx)
				e2Idx <- e2Idx + 1
			}
			geneIds[vIdx] <- paste(ens$name[gIdx], collapse=":")
			vIdx <- vIdx + 1
		}
	}
	if ((vIdx %% 100000) == 0)
	{
		print("+")
	}
}
vcfSel <- vcfSel[promFilter,]
print("Finished filtering of variants in protein coding gene promoters.")


gts <- extract.gt(vcfSel)
rm(vcfSel)
snpIDs <- rownames(gts)

# introduce binary dummy variables for haps
gtsMapped <- matrix(F, nrow=ncol(gts), ncol=nrow(gts)*4)
invisible(sapply(1:ncol(gts),function (i)
{
	sapply(1:nrow(gts), function(j)
	{
		s <- gts[j,i]
		if (s == "0|0")
		{
			gtsMapped[i,(j-1)*4 + 1] <<- T
		} else if (s == "1|0")
		{
			gtsMapped[i,(j-1)*4 + 2] <<- T
		} else if (s == "0|1")
		{
			gtsMapped[i,(j-1)*4 + 3] <<- T
		} else if (s == "1|1")
		{
			gtsMapped[i,(j-1)*4 + 4] <<- T
		} else {
			print(paste("WARNING: unknown haplo", s, "at position", i, j))

		}
	})
}))


colnames(gtsMapped) <- c(t(outer(snpIDs, c("0|0", "1|0", "0|1", "1|1"), function(x,y){paste(x,y,sep=".")})))
rownames(gtsMapped) <- colnames(gts)

# drop common variant
gtsMapped <- gtsMapped[,-seq(1,ncol(gtsMapped),4)]
filter <- colSums(gtsMapped) > 0
geneIds <- rep(geneIds[promFilter], each=3)[filter]
# drop haps not appearing
gtsMapped <- gtsMapped[, filter]


# convert database to .dat format
m <- lapply(
        1:nrow(gtsMapped),
        function (idx)
        {
            return(as.character(which(gtsMapped[idx,] == T)))
        }
    )

# get maximum number of ones in a row
maxCols <- max(
    sapply(1:length(m),
    function (idx)
    {
        return(length(m[[idx]]))
    }
    )
)
output_file1 <- paste(args[3], ".dat", sep="")
# flush output file
close(file(output_file1, open="w"))
# write result to specified file
invisible(lapply(m, write, output_file1, append=TRUE, ncolumns=maxCols, sep=" "))

write(colnames(gtsMapped), file=paste(args[3], ".colnames", sep=""))

write(rownames(gtsMapped), file=paste(args[3], ".rownames", sep=""))

write(geneIds, file=paste(args[3], ".genes", sep=""))
